/**
 * number_of_methods_and_constructors(+ClassId,-NOM)  
 * 
 * number of methods (that are no trivial getter methods)
 * Constructors are counted in this computation.
 * It will bind NOM to the number of methods of the class with ClassId.
 */
number_of_methods_and_constructors(ClassId,NOM):-
	count(MethodId,
          (type_contains_method(ClassId,MethodId),
           not(super_constructor(MethodId)),
           not(trivial_getter_method(MethodId))),
          NOM).

/**
 * number_of_methods_and_constructors_accessing_field(+FieldId,-NOMAF)  
 *
 * Binds the number of methods that access the field 
 * specified by FieldId.
 *
 */
number_of_methods_and_constructors_accessing_field(FieldId,NOMAF):-
    not(constant(FieldId)),
    field_is_in_type(FieldId, ClassId),
    count(Method,
          (type_contains_method(ClassId, Method),
           not(trivial_getter_method(Method)),
           once((ri_getFieldT_encl(Method, GetId),
                 getFieldT(GetId,_,_,_,_,FieldId)))),
          NOMAF).

/**
 * for_all_fields__sum_number_of_methods_and_constructors_accessing_field(+ClassId, -Result)
 */
for_all_fields__sum_number_of_methods_and_constructors_accessing_field(ClassId, Result):-
    findall(NOMAF, 
            (fieldT(FieldId, ClassId, _, _, _), 
             not(constant(FieldId)),
             number_of_methods_and_constructors_accessing_field(FieldId, NOMAF)), 
            ResultList),
    sumall(ResultList, Result).