:- multifile query_description/3.
:- multifile metric_/3.
:- multifile metric_attributes/2.

query_description(staticTestCoverage, metric, 'percentage of tested methods').
metric_attributes(staticTestCoverage, float(range(0,100), defaultthresholds(80,100))).


metric_(staticTestCoverage, ClassID, ReturnValue) :-
    packageT(PackageID,_),
    ri_compilationUnitT_parent(PackageID, CompilationUnit),
	ri_classT_parent(CompilationUnit, ClassID),
	not(externT(ClassID)),
	staticTestCoverage(ClassID, ReturnValue).

/**
  staticTestCoverage(+ClassId, -Percentage, -ID)
  
  Gives the percentage of public methods that are tested by some test class
  and if there are public untested methods, it will return one ID of these.
  if there is none, ID will be -1
*/ 
staticTestCoverage(ClassID, Percentage) :-
	not(test_class(ClassID)),
	findall(CalledMethodID, 
			(
			 	ri_methodT_parent(ClassID,CalledMethodID),
			 	methodDef(CalledMethodID, ClassID, MethodName, _, _, _, _),
			 	not(MethodName = '<init>'),
    		 	modifierT(CalledMethodID, 'public'),
    		 	method_is_called_by_test(CalledMethodID)
    		),
    		CalledMethodList),  
	length(CalledMethodList, NumberOfMethodsCalled),
	
	findall(PublicMethodID, 
			(
			 	ri_methodT_parent(ClassID,PublicMethodID),
				methodDef(PublicMethodID, ClassID, MethodName, _, _, _, _), 
			 	not(MethodName = '<init>'),
			 	modifierT(PublicMethodID, 'public')
			),
			PublicMethodList),
	length(PublicMethodList, NumberOfPublicMethods),
	
	%This will let this predicate fail, fi there is no method to be tested!
	not(NumberOfPublicMethods = 0),
	percentage(NumberOfMethodsCalled,NumberOfPublicMethods,Percentage)
	.
	%TODO what should this thing do? is_not_in_list(PublicMethodList, CalledMethodList, ID).
		