/**
 * extends_recursive(?SubClass,?SuperClass)
 *
 * Tests whether SubClass extends a SuperClass are another class
 * which recursively extends SuperClass.
 */
extends_recursive(SubClass,SuperClass) :- extendsRecursive(SubClass, SuperClass).
extends_recursive(SubClass,SuperClass,Count) :- extendsRecursive(SubClass, SuperClass,Count).
extends_recursive(SubClass,SuperClass,Init,Count) :- extendsRecursive(SubClass, SuperClass, Init, Count).
extends_recursive_from_subclass(SubClass,SuperClass) :- extendsRecursiveFromSubClass(SubClass,SuperClass).
extends_recursive_from_superclass(SubClass,SuperClass) :- extendsRecursiveFromSuperClass(SubClass,SuperClass).
% find all superclasses of subclass
extendsRecursive(SubClass, SuperClass) :-
    ((nonvar(SubClass),var(SuperClass));(nonvar(SubClass),nonvar(SuperClass))),
    extendsRecursiveFromSubClass(SubClass,SuperClass).

% find all subclasses of superclass
extendsRecursive(SubClass,SuperClass) :-
    var(SubClass),nonvar(SuperClass),
    extendsRecursiveFromSuperClass(SubClass,SuperClass).

% find all extension pairs    
extendsRecursive(SubClass,SuperClass) :-
    var(SubClass),var(SuperClass),
    classT(SubClass,_,_,_),
    extendsRecursiveFromSubClass(SubClass,SuperClass).

/**
 * extends_recursive_from_subclass(+SubClass,?SuperClass)
 */
extendsRecursiveFromSubClass(SubClass,SuperClass) :-
    extendsT(SubClass,SuperClass).

extendsRecursiveFromSubClass(SubClass,SuperClass) :-
    extendsT(SubClass,MiddleClass),
    not(MiddleClass=SuperClass),
    extendsRecursiveFromSubClass(MiddleClass, SuperClass).

/**
 * extendsRecursiveFromSuperClass(?SubClass,+SuperClass)
 */
extendsRecursiveFromSuperClass(SubClass,SuperClass) :-
%    extendsT_In(SuperClass,SubClass).
	extendsT(SubClass,SuperClass).

extendsRecursiveFromSuperClass(SubClass,SuperClass) :-
%    extendsT_In(SuperClass,MiddleClass),
    extendsT(MiddleClass,SuperClass),
    extendsRecursiveFromSuperClass(SubClass, MiddleClass).
    


% TODO refactor
/**
 * extendsRecursive(?+SubClass, ?SuperClass, ?Count)
 *    
 * Count is the number of inheritance steps from SubClass to SuperClass.
 */
extendsRecursive(SubClass, SuperClass,Count) :-
    Init = 0,
    extendsRecursive(SubClass, SuperClass,Init,Count).

extendsRecursive(SubClass, SuperClass, Init, Count) :-
    extendsT(SubClass, SuperClass),
    Count is Init+1.

extendsRecursive(SubClass, SuperClass, Init, Count) :-
        extendsT(SubClass, MiddleClass),
        extendsRecursive(MiddleClass, SuperClass, Init+1, Count).
        
        
