/**
*  semantic_scope(+MethodID, -ScopeValue) /2
*  succeeds iff ScopeValue is the semantic scope of method with ID MethodID
*   access:
    - private   : Value = 1
    - package   : Value = |Types in Package|
    - protected : Value = |Types in Package| + | Number of Children |
    - public    : Value = |Number of Types |
*
*/
semantic_scope(MethodID, ScopeValue) :-
         		visibility(MethodID, Access),
         		semantic_scope__(MethodID,Access,ScopeValue).
/**
*  semantic_scope__(+MethodID, +Access , -ScopeValue) /3
*  succeeds iff ScopeValue is the semantic scope of Arg1 with access Arg2
* @See semantic_scope/2
**/
semantic_scope__(MethodID, private , 1):- methodT(MethodID,_, _, _, _, _, _).

semantic_scope__(MethodID, package , NumberOfTypes):-
      									      methodT(MethodID,Type, _, _, _, _, _),
      									      type_is_in_package(Type, Package), 
                                              number_of_types(Package, NumberOfTypes).
                                                                               
semantic_scope__(MethodID, protected , Length):-
                                              methodT(MethodID,Type, _, _, _, _, _),
      									      type_is_in_package(Type, Package), 
                                              number_of_types(Package, NumberOfTypes),
                                              count(SubClassId,extendsRecursive(SubClassId, Type),NumberOfDescendants),
                                              Length is NumberOfTypes + NumberOfDescendants.
                                              
semantic_scope__(_MethodID, public , NumberOfAllInternalTypes):- 
											        count( 
												         Type,
												         internal_type(Type), 
												         NumberOfAllInternalTypes
											        ).
											        
											        
/**
*  semantic_scope(+MethodID, -ScopeValue) /2
*  succeeds iff ScopeValue is the semantic scope of method with ID MethodID
*   access:
    - private   : Value = 1
    - package   : Value = |Types in Package|
    - protected : Value = |Types in Package| + | Number of Children |
    - public    : Value = |Number of Types |
*
*/
semantic_scope_variant(MethodID, ScopeValue) :-
         		visibility(MethodID, Access),
         		semantic_scope_variant__(MethodID,Access,ScopeValue).
/**
*  semantic_scope__(+MethodID, +Access , -ScopeValue) /3
*  succeeds iff ScopeValue is the semantic scope of Arg1 with access Arg2
* @See semantic_scope/2
**/
semantic_scope_variant__(MethodID, private , Size):- 
											  methodT(MethodID,Type, _, _, _, _, _),
											  size_of_type(Type, Size).

semantic_scope_variant__(MethodID, package , SumSize):-
      									      methodT(MethodID,Type, _, _, _, _, _),
      									      type_is_in_package(Type, Package), 
      									      findall(Size, 
      									      			( package_contains_type(Package,Ty), size_of_type(Ty,Size)),
      									      			 ListSizes),
      									      sumlist(ListSizes, SumSize).
                                                                               
semantic_scope_variant__(MethodID, protected , Length):-
                                              methodT(MethodID,Type, _, _, _, _, _),
      									      type_is_in_package(Type, Package), 
                                              number_of_types(Package, NumberOfTypes),
                                              count(SubClassId,extendsRecursive(SubClassId, Type),NumberOfDescendants),
                                              Length is NumberOfTypes + NumberOfDescendants.
                                              
semantic_scope_variant__(_MethodID, public , NumberOfAllInternalTypes):- 
											        findall(Size,
												         (  internal_type(Type),size_of_type(Type,Size) ), 
												         SizeOfAllTypes
											        ),
											        sumlist(SizeOfAllTypes,NumberOfAllInternalTypes).
											        

	
					        